from collections import OrderedDict
from node.utils import UNSET
from yafowil.base import factory
from yafowil.common import generic_extractor, display_proxy_renderer
from yafowil.common import generic_required_extractor
from yafowil.common import input_generic_renderer
from yafowil.common import select_display_renderer
from yafowil.common import select_edit_renderer
from yafowil.common import select_extractor
from yafowil.utils import attr_value
from yafowil.utils import data_attrs_helper
from yafowil.utils import managedprops
from yafowil.utils import vocabulary


select2_options = [
    'width',
    'minimumInputLength',
    'maximumInputLength',
    'minimumResultsForSearch',
    'maximumSelectionSize',
    'placeholder',
    'placeholderOption',
    'separator',
    'allowClear',
    'multiple',
    'closeOnSelect',
    'openOnEnter',
    'id',
    'matcher',
    'sortResults',
    'formatSelection',
    'formatResult',
    'formatResultCssClass',
    'formatNoMatches',
    'formatSearching',
    'formatInputTooShort',
    'formatInputTooLong',
    'formatSelectionTooBig',
    'formatLoadMore',
    'createSearchChoice',
    'createSearchChoicePosition',
    'initSelection',
    'tokenizer',
    'tokenSeparators',
    'query',
    'ajax',
    'data',
    'tags',
    'containerCss',
    'containerCssClass',
    'dropdownCss',
    'dropdownCssClass',
    'dropdownAutoWidth',
    'adaptContainerCssClass',
    'adaptDropdownCssClass',
    'escapeMarkup',
    'selectOnBlur',
    'loadMorePadding',
    'nextSearchTerm',
]


@managedprops('inputtag')
def select2_extractor(widget, data):
    if attr_value('inputtag', widget, data):
        extracted = generic_extractor(widget, data)
        if extracted is UNSET:
            return extracted
        if not extracted:
            if attr_value('multiple', widget, data):
                return []
            return ''
        if attr_value('inputtag', widget, data):
            if attr_value('multiple', widget, data):
                extracted = extracted.split(',')
        return extracted
    return select_extractor(widget, data)


@managedprops('inputtag', 'ajaxurl', *select2_options)
def select2_edit_renderer(widget, data):
    if attr_value('inputtag', widget, data):
        extracted = UNSET
        reset_extracted = False
        if data.extracted is not UNSET:
            extracted = data.extracted
            if isinstance(extracted, list) or isinstance(extracted, tuple):
                reset_extracted = True
                data.extracted = u','.join(extracted)
        value = UNSET
        reset_value = False
        if data.value is not UNSET:
            value = data.value
            if isinstance(value, list) or isinstance(value, tuple):
                reset_value = True
                data.value = u','.join(value)
        else:
            reset_value = True
            data.value = u''
        # force widget vocabulary as dict to make JS happy
        vocab = vocabulary(attr_value('vocabulary', widget, data, []))
        if vocab:
            # Use ordered dict to ensure order in tests
            dict_vocab = OrderedDict()
            for key, term in vocab:
                dict_vocab[key] = term
            widget.attrs['vocabulary'] = dict_vocab
        custom_attrs = data_attrs_helper(
            widget, data, ['ajaxurl', 'vocabulary'] + select2_options)
        renderer = input_generic_renderer
        rendered = renderer(widget, data, custom_attrs=custom_attrs)
        if reset_extracted:
            data.extracted = extracted
        if reset_value:
            data.value = value
    else:
        multiple = attr_value('multiple', widget, data)
        if multiple:
            widget.attrs['multivalued'] = True
            del widget.attrs['multiple']
        custom_attrs = data_attrs_helper(
            widget, data, ['ajaxurl'] + select2_options)
        renderer = select_edit_renderer
        rendered = renderer(widget, data, custom_attrs=custom_attrs)
    return rendered


def select2_display_renderer(widget, data):
    multiple = attr_value('multiple', widget, data)
    multivalued = attr_value('multivalued', widget, data)
    if multiple and not multivalued:
        widget.attrs['multivalued'] = True
    return select_display_renderer(widget, data)


factory.register(
    'select2',
    extractors=[select2_extractor, generic_required_extractor],
    edit_renderers=[select2_edit_renderer],
    display_renderers=[
        select2_display_renderer,
        display_proxy_renderer
    ])

factory.doc['blueprint']['select2'] = """\
Add-on blueprint
`yafowil.widget.select2 <http://github.com/conestack/yafowil.widget.select2/>`_

`Integrates Select2 Widget <http://ivaynberg.github.io/select2/>`_

`Detailed widget documentation <http://ivaynberg.github.io/select2/#documentation>`_
"""

factory.defaults['select2.size'] = None
factory.defaults['select2.default'] = []
factory.defaults['select2.format'] = 'block'

factory.defaults['select2.class'] = 'select2'

factory.defaults['select2.inputtag'] = False
factory.doc['props']['select2.inputtag'] = """\
Render widget as input element instead of selection.
"""

factory.defaults['select2.ajaxurl'] = None
factory.doc['props']['select2.ajaxurl'] = """\
Ajax URL to JSON view returning an array of objects like::

    [{id: 'id', text: 'text'}]
"""

factory.defaults['select2.width'] = None
factory.doc['props']['select2.width'] = """\
Controls the width style attribute of the Select2 container div.
The following values are supported:

off
    No width attribute will be set. Keep in mind that the container div copies
    classes from the source element so setting the width attribute may not
    always be necessary.

element
    Uses javascript to calculate the width of the source element.

copy
    Copies the value of the width style attribute set on the source element.

resolve
    First attempts to copy than falls back on element.

other values
    if the width attribute contains a function it will be evaluated, otherwise
    the value is used verbatim.
"""

factory.defaults['select2.minimumInputLength'] = None
factory.doc['props']['select2.minimumInputLength'] = """\
Number of characters necessary to start a search.
"""

factory.defaults['select2.maximumInputLength'] = None
factory.doc['props']['select2.maximumInputLength'] = """\
Maximum number of characters that can be entered for an input.
"""

factory.defaults['select2.minimumResultsForSearch'] = None
factory.doc['props']['select2.minimumResultsForSearch'] = """\
The minimum number of results that must be initially (after opening the
dropdown for the first time) populated in order to keep the search field.
This is useful for cases where local data is used with just a few results,
in which case the search box is not very useful and wastes screen space.

The option can be set to a negative value to permanently hide the search field.

Only applies to single-value select boxes.
"""

factory.defaults['select2.maximumSelectionSize'] = None
factory.doc['props']['select2.maximumSelectionSize'] = """\
The maximum number of items that can be selected in a multi-select control.
If this number is less than 1 selection is not limited.

Once the number of selected items reaches the maximum specified the contents
of the dropdown will be populated by the formatSelectionTooBig function.
"""

factory.defaults['select2.placeholder'] = None
factory.doc['props']['select2.placeholder'] = """\
Initial value that is selected if no other selection is made.

The placeholder can also be specified as a data-placeholder attribute on the
select or input element that Select2 is attached to.

Note that because browsers assume the first option element is selected in
non-multi-value select boxes an empty first option element must be provided
(<option></option>) for the placeholder to work.
"""

factory.defaults['select2.placeholderOption'] = None
factory.doc['props']['select2.placeholderOption'] = """\
When attached to a select resolves the option that should be used as the
placeholder. Can either be a function which given the select element should
return the option element or a string first to indicate that the first option
should be used.

This option is useful when Select2's default of using the first option only if
it has no value and no text is not suitable.
"""

factory.defaults['select2.separator'] = None
factory.doc['props']['select2.separator'] = """\
Separator character or string used to delimit ids in value attribute of the
multi-valued selects. The default delimiter is the , character.
"""

factory.defaults['select2.allowClear'] = None
factory.doc['props']['select2.allowClear'] = """\
Whether or not a clear button is displayed when the select box has a
selection. The button, when clicked, resets the value of the select box back
to the placeholder, thus this option is only available when the placeholder is
specified.

This option only works when the placeholder is specified.

When attached to a select an option with an empty value must be provided. This
is the option that will be selected when the button is pressed since a select
box requires at least one selection option.

Also, note that this option only works with non-multi-value based selects
because multi-value selects always provide such a button for every selected
option.
"""

factory.defaults['select2.multiple'] = None
factory.doc['props']['select2.multiple'] = """\
Whether or not Select2 allows selection of multiple values.

When Select2 is attached to a select element this value will be ignored and
select's multiple attribute will be used instead.
"""

factory.defaults['select2.closeOnSelect'] = None
factory.doc['props']['select2.closeOnSelect'] = """\
If set to false the dropdown is not closed after a selection is made,
allowing for rapid selection of multiple items. By default this option is set
to true.

Only applies when configured in multi-select mode.
"""

factory.defaults['select2.openOnEnter'] = None
factory.doc['props']['select2.openOnEnter'] = """\
If set to true the dropdown is opened when the user presses the enter key
and Select2 is closed. By default this option is enabled.
"""

factory.defaults['select2.id'] = None
factory.doc['props']['select2.id'] = """\
Function used to get the id from the choice object or a string representing
the key under which the id is stored::

    id(object)

The default implementation expects the object to have a id property that is
returned.
"""

factory.defaults['select2.matcher'] = None
factory.doc['props']['select2.matcher'] = """\
Used to determine whether or not the search term matches an option when a
built-in query function is used. The built in query function is used when
Select2 is attached to a select, or the local or tags helpers are used::

    matcher(term, text, option)

The default implementation is case insensitive and matches anywhere in the
term::

    function(term, text) {
        return text.toUpperCase().indexOf(term.toUpperCase()) >= 0;
    }
"""

factory.defaults['select2.sortResults'] = None
factory.doc['props']['select2.sortResults'] = """\
Used to sort the results list for searching right before display.
Useful for sorting matches by relevance to the user's search term::

    sortResults(results, container, query)

Defaults to no sorting::

    function(results, container, query) {
        return results;
    }
"""

factory.defaults['select2.formatSelection'] = None
factory.doc['props']['select2.formatSelection'] = """\
Function used to render the current selection::

    formatSelection(object, container)

The default implementation expects the object to have a text property that is
returned.

The implementation may choose to append elements directly to the provided
container object, or return a single value and have it automatically appended.

When attached to a select the original <option> (or <optgroup>) element is
accessible inside the specified function through the property item.element::

    format(item) {
        var originalOption = item.element;
        return item.text
    }
"""

factory.defaults['select2.formatResult'] = None
factory.doc['props']['select2.formatResult'] = """\
Function used to render a result that the user can select::

    formatResult(object, container, query)

The default implementation expects the object to have a text property that is
returned.

The implementation may choose to append elements directly to the provided
container object, or return a single value and have it automatically appended.

When attached to a select the original <option> (or <optgroup>) element is
accessible inside the specified function through the property item.element::

    format(item) {
        var originalOption = item.element;
        return item.text
    }
"""

factory.defaults['select2.formatResultCssClass'] = None
factory.doc['props']['select2.formatResultCssClass'] = """\
Function used to add css classes to result elements::

    formatResultCssClass(object)

By default when attached to a select css classes from options will be
automatically copied.
"""

factory.defaults['select2.formatNoMatches'] = None
factory.doc['props']['select2.formatNoMatches'] = """\
String containing "No matches" message, or
Function used to render the message::

    formatNoMatches(term)
"""

factory.defaults['select2.formatSearching'] = None
factory.doc['props']['select2.formatSearching'] = """\
String containing "Searching..." message, or
Function used to render the message that is displayed while search is in
progress::

    formatSearching()
"""

factory.defaults['select2.formatInputTooShort'] = None
factory.doc['props']['select2.formatInputTooShort'] = """\
String containing "Search input too short" message, or Function used to
render the message::

    formatInputTooShort(term, minLength)
"""

factory.defaults['select2.formatInputTooLong'] = None
factory.doc['props']['select2.formatInputTooLong'] = """\
String containing "Search input too long" message, or Function used to
render the message::

    formatInputTooLong(term, maxLength)
"""

factory.defaults['select2.formatSelectionTooBig'] = None
factory.doc['props']['select2.formatSelectionTooBig'] = """\
String containing "You cannot select any more choices" message, or Function
used to render the message::

    formatSelectionTooBig(maxSize)
"""

factory.defaults['select2.formatLoadMore'] = None
factory.doc['props']['select2.formatLoadMore'] = """\
String containing "Loading more results" message, or Function used to
render the message::

    formatLoadMore(pageNumber)
"""

factory.defaults['select2.createSearchChoice'] = None
factory.doc['props']['select2.createSearchChoice'] = """\
Creates a new selectable choice from user's search term. Allows creation of
choices not available via the query function. Useful when the user can create
choices on the fly, eg for the 'tagging' usecase::

    createSearchChoice(term)

If the function returns undefined or null no choice will be created. If a new
choice is created it is displayed first in the selection list so that user
may select it by simply pressing enter.

When used in combination with input[type=hidden] tag care must be taken to
sanitize the id attribute of the choice object, especially stripping , as it
is used as a value separator.
"""

factory.defaults['select2.createSearchChoicePosition'] = None
factory.doc['props']['select2.createSearchChoicePosition'] = """\
Define the position where to insert element created by createSearchChoice.
The following values are supported:

top
    Insert in the top of the list

bottom
    Insert at the end of the list

function
    A custom function. For example if you want to insert the new item in the
    second position::

        $("#tags").select2({
            ...
            createSearchChoice: function(term) { ... },
            createSearchChoicePosition: function(list, item) {
                list.splice(1, 0, item);
            }
        });
"""

factory.defaults['select2.initSelection'] = None
factory.doc['props']['select2.initSelection'] = """\
Called when Select2 is created to allow the user to initialize the
selection based on the value of the element select2 is attached to.

Essentially this is an id->object mapping function::

    initSelection(element, callback)

This function will only be called when there is initial input to be processed.
"""

factory.defaults['select2.tokenizer'] = None
factory.doc['props']['select2.tokenizer'] = """\
A tokenizer function can process the input typed into the search field after
every keystroke and extract and select choices. This is useful, for example,
in tagging scenarios where the user can create tags quickly by separating
them with a comma or a space instead of pressing enter.

Tokenizer only applies to multi-selects::

    tokenizer(input, selection, selectCallback, opts)
"""

factory.defaults['select2.tokenSeparators'] = None
factory.doc['props']['select2.tokenSeparators'] = """\
An array of strings that define token separators for the default tokenizer
function. By default, this option is set to an empty array which means
tokenization using the default tokenizer is disabled. Usually it is sensible
to set this option to a value similar to [',', ' '].
"""

factory.defaults['select2.query'] = None
factory.doc['props']['select2.query'] = """\
Function used to query results for the search term::

    query(options)

In order for this function to work Select2 should be attached to a input
type='hidden' tag instead of a select.
"""

factory.defaults['select2.ajax'] = None
factory.doc['props']['select2.ajax'] = """\
Options for the built in ajax query function. This object acts as a
shortcut for having to manually write a function that performs ajax requests.
The built-in function supports more advanced features such as throttling and
dropping out-of-order responses.

In order for this function to work Select2 should be attached to a input
type='hidden' tag instead of a select.
"""

factory.defaults['select2.data'] = None
factory.doc['props']['select2.data'] = """\
Options for the built in query function that works with arrays.

If this element contains an array, each element in the array must contain id
and text keys.

Alternatively, this element can be specified as an object in which results
key must contain the data as an array and a text key can either be the name
of the key in data items that contains text or a function that retrieves
the text given a data element from the array.
"""

factory.defaults['select2.tags'] = None
factory.doc['props']['select2.tags'] = """\
Puts Select2 into 'tagging' mode where the user can add new choices and
pre-existing tags are provided via this options attribute which is either an
array or a function that returns an array of objects or strings. If strings
are used instead of objects they will be converted into an object that has
an id and text attribute equal to the value of the string.
"""

factory.defaults['select2.containerCss'] = None
factory.doc['props']['select2.containerCss'] = """\
Inline css that will be added to select2's container. Either an object
containing css property/value key pairs or a function that returns such an
object.
"""

factory.defaults['select2.containerCssClass'] = None
factory.doc['props']['select2.containerCssClass'] = """\
Css class that will be added to select2's container tag.
"""

factory.defaults['select2.dropdownCss'] = None
factory.doc['props']['select2.dropdownCss'] = """\
Inline css that will be added to select2's dropdown container. Either an
object containing css property/value key pairs or a function that returns such
an object.
"""

factory.defaults['select2.dropdownCssClass'] = None
factory.doc['props']['select2.dropdownCssClass'] = """\
Css class that will be added to select2's dropdown container.
"""

factory.defaults['select2.dropdownAutoWidth'] = None
factory.doc['props']['select2.dropdownAutoWidth'] = """\
When set to true attempts to automatically size the width of the dropdown
based on content inside.
"""

factory.defaults['select2.adaptContainerCssClass'] = None
factory.doc['props']['select2.adaptContainerCssClass'] = """\
Function that filters/renames css classes as they are copied from the
source tag to the select2 container tag::

    adaptContainerCssClass(clazz)

The default implementation applies all classes without modification.
"""

factory.defaults['select2.adaptDropdownCssClass'] = None
factory.doc['props']['select2.adaptDropdownCssClass'] = """\
Function that filters/renames css classes as they are copied from the
source tag to the select2 dropdown tag::

    adaptDropdownCssClass(clazz)

The default implementation always returns null thereby filtering out all
classes.
"""

factory.defaults['select2.escapeMarkup'] = None
factory.doc['props']['select2.escapeMarkup'] = """\
String escapeMarkup(String markup)

Function used to post-process markup returned from formatter functions. By
default this function escapes html entities to prevent javascript injection.
"""

factory.defaults['select2.selectOnBlur'] = None
factory.doc['props']['select2.selectOnBlur'] = """\
Set to true if you want Select2 to select the currently highlighted option
when it is blurred.
"""

factory.defaults['select2.loadMorePadding'] = None
factory.doc['props']['select2.loadMorePadding'] = """\
Defines how many pixels need to be below the fold before the next page is
loaded. The default value is 0 which means the result list needs to be
scrolled all the way to the bottom for the next page of results to be loaded.
This option can be used to trigger the load sooner, possibly resulting in a
smoother user experience.
"""

factory.defaults['select2.nextSearchTerm'] = None
factory.doc['props']['select2.nextSearchTerm'] = """\
Function used to determine what the next search term should be.

Function can be used when the dropdown is configured in single and
multi-select mode. It is triggered after selecting an item. In single mode
it is also triggered after initSelection (when provided).
"""



from yafowil.base import factory
from yafowil.utils import entry_point
import os


resourcedir = os.path.join(os.path.dirname(__file__), 'resources')
js = [{
    'group': 'yafowil.widget.select2.dependencies',
    'resource': 'select2/select2.js',
    'order': 20,
}, {
    'group': 'yafowil.widget.select2.common',
    'resource': 'widget.js',
    'order': 21,
}]
css = [{
    'group': 'yafowil.widget.select2.dependencies',
    'resource': 'select2/select2.css',
    'order': 20,
}, {
    'group': 'yafowil.widget.select2.common',
    'resource': 'widget.css',
    'order': 21,
}]


@entry_point(order=10)
def register():
    from yafowil.widget.select2 import widget  # noqa
    factory.register_theme('default', 'yafowil.widget.select2',
                           resourcedir, js=js, css=css)



# -*- coding: utf-8 -*-
from yafowil.base import factory
from yafowil.compat import IS_PY2
import json


if IS_PY2:
    from urlparse import urlparse
    from urlparse import parse_qs
else:
    from urllib.parse import urlparse
    from urllib.parse import parse_qs


def json_response(url):
    purl = urlparse(url)
    qs = parse_qs(purl.query)
    data = json_data(qs.get('q', [''])[0])
    return {
        'body': json.dumps(data),
        'header': [('Content-Type', 'application/json')]
    }


def json_data(term):
    vocab = sorted((u'Weißburgunder', u'Welschriesling',
                    u'Sauvingnon Blanc', u'Sämling', u'Scheurebe',
                    u'Traminer', u'Morrilon', u'Muskateller'))
    data = list()
    if term:
        for val in vocab:
            if val.lower().startswith(term.lower()):
                data.append({
                    'id': val,
                    'text': val,
                })
    print(term)
    print(data)
    return data


TITLE_SELECT2_1 = "Single selection"
DOC_SELECT2_1 = """
select2
-------

Select2 widget in single selection mode.

.. code-block:: python

    vocab = sorted((u'Weißburgunder', u'Welschriesling',
                    u'Sauvingnon Blanc', u'Sämling', u'Scheurebe',
                    u'Traminer', u'Morrilon', u'Muskateller'))

    select2 = factory('#field:select2', value='Traminer', props={
        'label': 'Select some items',
        'placeholder': 'Select some items',
        'vocabulary': vocab,
    })

"""

TITLE_SELECT2_2 = "Multi selection"
DOC_SELECT2_2 = """
Select2 widget in multi selection mode.

.. code-block:: python

    value = [u'Sauvingnon Blanc', u'Sämling']
    select2 = factory('#field:select2', value=value, props={
        'label': 'Select some items',
        'placeholder': 'Select some items',
        'vocabulary': vocab,
        'multiple': True,
    })

"""

TITLE_SELECT2_3 = "Tagging Mode"
DOC_SELECT2_3 = """
Select2 widget in tag mode.

.. code-block:: python

    value = [u'Sämling', u'Scheurebe']
    select2 = factory('#field:select2', value=value, props={
        'label': 'Select or add some items',
        'inputtag': True,
        'placeholder': 'Select or add some items',
        'minimumInputLength': 1,
        'multiple': True,
        'tags': vocab,
    })

"""


TITLE_SELECT2_4 = "Single selection from ajax URL"
DOC_SELECT2_4 = """
Select2 single selection widget fetching data from URL.

.. code-block:: python

    select2 = factory('#field:select2', value='Welschriesling', props={
        'label': 'Select ajax item',
        'inputtag': True,
        'placeholder': 'Select some item',
        'ajaxurl': 'yafowil.widget.select2.json',
    })

The server answers with a JSON response, here the example does it using WSGI
and ``webob`` way. This code needs modification depending on the framework
used

.. code-block:: python

    def json_response(environ, start_response):
        data = lipsum
        if environ['QUERY_STRING'].startswith('q='):
            term = environ['QUERY_STRING'][3:]
            data = list()
            if term:
                for val in vocab:
                    if val.lower().startswith(term.lower()):
                        data.append({
                            'id': val,
                            'text': val,
                        })
        response = Response(content_type='application/json',
                            body=json.dumps(data))
        return response(environ, start_response)

"""


TITLE_SELECT2_5 = "Multi selection from ajax URL"
DOC_SELECT2_5 = """
Select2 multi selection widget fetching data from URL.

.. code-block:: python

    value = [u'Sämling', u'Traminer']
    select2 = factory('#field:select2', value=value, props={
        'label': 'Select multiple ajax items',
        'inputtag': True,
        'minimumInputLength': 1,
        'multiple': True,
        'placeholder': 'Select some items',
        'ajaxurl': 'yafowil.widget.select2.json',
    })

"""


TITLE_SELECT2_6 = "Tagging mode with ajax"
DOC_SELECT2_6 = """
Select2 tagging selection widget fetching data from URL. Also provide a
vocabulary defining terms for values.

.. code-block:: python

    value = ['a', 'b', 'c']
    vocabulary = [('a', u'Sämling'),
                  ('b', u'Traminer'),
                  ('c', u'Welschriesling')]
    select2 = factory('#field:select2', value=value, props={
        'label': 'Select tags',
        'vocabulary': vocabulary,
        'inputtag': True,
        'minimumInputLength': 1,
        'multiple': True,
        'placeholder': 'Select tags',
        'ajaxurl': 'yafowil.widget.select2.json',
        'tags': True,
    })

"""


def get_example():

    vocab = sorted((u'Weißburgunder', u'Welschriesling',
                    u'Sauvingnon Blanc', u'Sämling', u'Scheurebe',
                    u'Traminer', u'Morrilon', u'Muskateller'))

    # single selection
    select2_1 = factory(u'fieldset', name='yafowil_select2_1')
    select2_1['text'] = factory('#field:select2', value='Traminer', props={
        'label': 'Select some items',
        'placeholder': 'Select some items',
        'vocabulary': vocab,
    })

    # multiple selection
    select2_2 = factory(u'fieldset', name='yafowil_select2_2')
    select2_2_val = [u'Sauvingnon Blanc', u'Sämling']
    select2_2['text'] = factory('#field:select2', value=select2_2_val, props={
        'label': 'Select some items',
        'placeholder': 'Select some items',
        'vocabulary': vocab,
        'multiple': True,
    })

    # input mode
    select2_3 = factory(u'fieldset', name='yafowil_select2_3')
    select2_3_val = [u'Sämling', u'Scheurebe']
    select2_3['text'] = factory('#field:select2', value=select2_3_val, props={
        'label': 'Select or add some items',
        'inputtag': True,
        'placeholder': 'Select or add some items',
        'minimumInputLength': 1,
        'multiple': True,
        'tags': vocab,
    })

    # ajax single
    select2_4 = factory(u'fieldset', name='yafowil_select2_4')
    select2_4_val = 'Welschriesling'
    select2_4['text'] = factory('#field:select2', value=select2_4_val, props={
        'label': 'Select ajax item',
        'inputtag': True,
        'placeholder': 'Select some item',
        'ajaxurl': 'yafowil.widget.select2.json',
    })
    select2_4_routes = {'yafowil.widget.select2.json': json_response}

    # ajax multiple
    select2_5 = factory(u'fieldset', name='yafowil_select2_5')
    select2_5_val = [u'Sämling', u'Traminer']
    select2_5['text'] = factory('#field:select2', value=select2_5_val, props={
        'label': 'Select multiple ajax items',
        'inputtag': True,
        'minimumInputLength': 1,
        'multiple': True,
        'placeholder': 'Select some items',
        'ajaxurl': 'yafowil.widget.select2.json',
    })
    select2_5_routes = {'yafowil.widget.select2.json': json_response}

    # ajax tagging
    select2_6 = factory(u'fieldset', name='yafowil_select2_6')
    select2_6_val = ['a', 'b', 'c']
    select2_6_vocab = [('a', u'Sämling'),
                       ('b', u'Traminer'),
                       ('c', u'Welschriesling')]
    select2_6['text'] = factory('#field:select2', value=select2_6_val, props={
        'label': 'Select tags',
        'vocabulary': select2_6_vocab,
        'inputtag': True,
        'minimumInputLength': 1,
        'multiple': True,
        'placeholder': 'Select tags',
        'ajaxurl': 'yafowil.widget.select2.json',
        'tags': True,
    })
    select2_6_routes = {'yafowil.widget.select2.json': json_response}

    return [{
        'widget': select2_1,
        'doc': DOC_SELECT2_1,
        'title': TITLE_SELECT2_1
    }, {
        'widget': select2_2,
        'doc': DOC_SELECT2_2,
        'title': TITLE_SELECT2_2
    }, {
        'widget': select2_3,
        'doc': DOC_SELECT2_3,
        'title': TITLE_SELECT2_3
    }, {
        'widget': select2_4,
        'routes': select2_4_routes,
        'doc': DOC_SELECT2_4,
        'title': TITLE_SELECT2_4
    }, {
        'widget': select2_5,
        'routes': select2_5_routes,
        'doc': DOC_SELECT2_5,
        'title': TITLE_SELECT2_5
    }, {
        'widget': select2_6,
        'routes': select2_6_routes,
        'doc': DOC_SELECT2_6,
        'title': TITLE_SELECT2_6
    }]



from collections import OrderedDict
from node.utils import UNSET
from yafowil.base import ExtractionError
from yafowil.base import factory
from yafowil.compat import IS_PY2
from yafowil.tests import YafowilTestCase
import unittest
import yafowil.loader  # noqa


if not IS_PY2:
    from importlib import reload


class TestSelect2Widget(YafowilTestCase):

    def setUp(self):
        super(TestSelect2Widget, self).setUp()
        from yafowil.widget.select2 import widget
        reload(widget)

    def test_render_single(self):
        widget = factory(
            'select2',
            name='single',
            props={
                'required': True
            })
        self.assertEqual(widget(), (
            '<select class="select2" id="input-single" name="single" '
            'required="required"> </select>'
        ))

    def test_render_multiple(self):
        widget = factory(
            'select2',
            name='multi',
            props={
                'required': True,
                'multiple': True
            })
        self.assertEqual(widget(), (
            '<input id="exists-multi" name="multi-exists" type="hidden" '
            'value="exists" /><select class="select2" id="input-multi" '
            'multiple="multiple" name="multi" required="required"> </select>'
        ))

    def test_render_single_tag_mode(self):
        widget = factory(
            'select2',
            name='single',
            props={
                'inputtag': True
            })
        self.assertEqual(widget(), (
            '<input class="select2" id="input-single" name="single" '
            'value="" />'
        ))

    def test_render_multiple_tag_mode(self):
        widget = factory(
            'select2',
            name='multi',
            value=['1', '2'],
            props={
                'inputtag': True,
                'multiple': True
            })
        self.assertEqual(widget(), (
            '<input class="select2" data-multiple=\'true\' id="input-multi" '
            'name="multi" value="1,2" />'
        ))

    def test_render_with_vocabulary(self):
        # Provide a vocabulary if value terms consists of id / label pairs
        vocab = OrderedDict()
        vocab['1'] = 'Label 1'
        vocab['2'] = 'Label 2'
        widget = factory(
            'select2',
            name='multi',
            value=['1', '2'],
            props={
                'inputtag': True,
                'multiple': True,
                'vocabulary': vocab,
            })
        self.assertEqual(widget(), (
            '<input class="select2" data-multiple=\'true\' '
            'data-vocabulary=\'{"1": "Label 1", "2": "Label 2"}\' '
            'id="input-multi" name="multi" value="1,2" />'
        ))

    def test_extract_required(self):
        widget = factory(
            'select2',
            name='single',
            props={
                'required': True
            })

        request = {}
        data = widget.extract(request)
        self.assertEqual(data.extracted, UNSET)

        request = {'single': ''}
        data = widget.extract(request)
        self.assertEqual(
            data.errors,
            [ExtractionError('Mandatory field was empty')]
        )
        self.assertEqual(data.extracted, '')

        request = {'single': '1'}
        data = widget.extract(request)
        self.assertEqual(data.errors, [])
        self.assertEqual(data.extracted, '1')

    def test_extract_single_value(self):
        widget = factory(
            'select2',
            name='single',
            props={
                'inputtag': True
            })

        request = {}
        data = widget.extract(request)
        self.assertEqual(data.extracted, UNSET)

        request = {'single': ''}
        data = widget.extract(request)
        self.assertEqual(data.extracted, '')

        request = {'single': '1'}
        data = widget.extract(request)
        self.assertEqual(data.extracted, '1')

    def test_extract_multi_value(self):
        widget = factory(
            'select2',
            name='multi',
            props={
                'multiple': True,
                'inputtag': True
            })

        request = {}
        data = widget.extract(request)
        self.assertEqual(data.extracted, UNSET)

        request = {'multi': ''}
        data = widget.extract(request)
        self.assertEqual(data.extracted, [])

        request = {'multi': '1,2'}
        data = widget.extract(request)
        self.assertEqual(data.extracted, ['1', '2'])

    def test_extract_with_preset_value(self):
        widget = factory(
            'select2',
            name='multi',
            value=['1', '2'],
            props={
                'multiple': True,
                'inputtag': True
            })
        self.assertEqual(widget(), (
            '<input class="select2" data-multiple=\'true\' id="input-multi" '
            'name="multi" value="1,2" />'
        ))

        request = {'multi': '1,2,3'}
        data = widget.extract(request)
        self.assertEqual(widget(data=data), (
            '<input class="select2" data-multiple=\'true\' id="input-multi" '
            'name="multi" value="1,2,3" />'
        ))
        self.assertEqual(data.extracted, ['1', '2', '3'])

        request = {'multi': ''}
        data = widget.extract(request)
        self.assertEqual(widget(data=data), (
            '<input class="select2" data-multiple=\'true\' id="input-multi" '
            'name="multi" value="" />'
        ))
        self.assertEqual(data.extracted, [])

    def test_display_renderer_empty(self):
        widget = factory(
            'select2',
            name='empty',
            mode='display')
        self.assertEqual(
            widget(),
            '<div class="display-select2" id="display-empty"></div>'
        )

    def test_display_renderer_single(self):
        widget = factory(
            'select2',
            name='single',
            value='foo',
            props={
                'vocabulary': [('foo', 'Foo'), ('bar', 'Bar')]
            },
            mode='display')
        self.assertEqual(
            widget(),
            '<div class="display-select2" id="display-single">Foo</div>'
        )

    def test_display_renderer_multiple(self):
        widget = factory(
            'select2',
            name='multi',
            value=['foo', 'bar'],
            props={
                'vocabulary': [('foo', 'Foo'), ('bar', 'Bar')],
                'multiple': True,
            },
            mode='display')
        self.assertEqual(widget(), (
            '<ul class="display-select2" '
            'id="display-multi"><li>Foo</li><li>Bar</li></ul>'
        ))

    # test with display_proxy_renderer
    def test_extract_with_preset_value_and_display_proxy_renderer(self):
        widget = factory(
            'select2',
            name='multi',
            value=['1', '2'],
            mode='display',
            props={
                'multiple': True,
                'inputtag': True,
                'display_proxy': True
            })
        self.assertEqual(widget(), (
            '<ul class="display-select2" id="display-multi"><li>1</li><li>2</li></ul>'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="1" />'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="2" />'
        ))

        request = {'multi': '1,2,3'}
        data = widget.extract(request)
        self.assertEqual(widget(data=data), (
            '<ul class="display-select2" id="display-multi"><li>1</li><li>2</li><li>3</li></ul>'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="1" />'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="2" />'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="3" />'
        ))
        self.assertEqual(data.extracted, ['1', '2', '3'])

        request = {'multi': ''}
        data = widget.extract(request)
        self.assertEqual(widget(data=data), (
            '<div class="display-select2" id="display-multi"></div>'
        ))
        self.assertEqual(data.extracted, [])

    def test_display_renderer_empty_and_display_proxy_renderer(self):
        widget = factory(
            'select2',
            name='empty',
            mode='display',
            props={
                'display_proxy': True
            }
        )
        self.assertEqual(
            widget(),
            '<div class="display-select2" id="display-empty"></div>'
        )

    def test_display_renderer_single_and_display_proxy_renderer(self):
        widget = factory(
            'select2',
            name='single',
            value='foo',
            mode='display',
            props={
                'vocabulary': [('foo', 'Foo'), ('bar', 'Bar')],
                'display_proxy': True
            },
        )
        self.assertEqual(
            widget(),
            '<div class="display-select2" id="display-single">Foo</div>'
            '<input class="select2" id="input-single" name="single" type="hidden" value="foo" />'
        )

    def test_display_renderer_multiple_and_display_proxy_renderer(self):
        widget = factory(
            'select2',
            name='multi',
            value=['foo', 'bar'],
            props={
                'vocabulary': [('foo', 'Foo'), ('bar', 'Bar')],
                'multiple': True,
                'display_proxy': True
            },
            mode='display')
        self.assertEqual(widget(), (
            '<ul class="display-select2" id="display-multi"><li>Foo</li><li>Bar</li></ul>'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="foo" />'
            '<input class="select2" id="input-multi" name="multi" type="hidden" value="bar" />'
        ))

if __name__ == '__main__':
    unittest.main()
